import React, { Component, PropTypes } from 'react'
import raf from 'raf'

export default class NavBackground extends Component {
  constructor (props) {
    super(props)

    this.state = {
      isOpaque: this.props.isOpaque,
      isOpen: this.props.isOpen,
      backgroundOpacity: 0.9
    }
  }

  componentDidMount () {
    window.addEventListener('scroll', () => {
      if (this.RafId) return

      this.RafId = raf(() => {
        if (this.props.isOpen) {
          this.RafId = undefined
          return
        }

        const { scrollY } = window
        const { isOpaque } = this.state

        if (scrollY >= 50 && !isOpaque) {
          this.setState({ isOpaque: true })
        } else if (!this.props.isOpaque && scrollY < 50 && isOpaque) {
          this.setState({ isOpaque: false })
        }

        this.RafId = undefined
      })
    })
  }

  componentWillReceiveProps (props) {
    const obj = {}

    if (props.isOpen) {
      obj.isOpen = props.isOpen
    }
    if (props.isOpaque) {
      obj.isOpaque = (window.scrollY < 50) ? props.isOpaque : true
    }

    this.setState(obj)

    if (!props.isOpen) return

    const listener = this.hasTouch ? 'touchend' : 'mouseup'
    const setActiveSection = this.props.setActiveSection

    window.addEventListener(listener, function onWindowPress (e) {
      let el = e.target
      let toTrigger = true

      while (el) {
        if (/nav/.test(el.className)) {
          toTrigger = false
          break
        }

        el = el.parentNode
      }

      if (toTrigger) {
        setActiveSection(-1)
        window.removeEventListener(listener, onWindowPress)
      }
    })
  }

  render () {
    let classnames = 'nav-background flex flex--center-x'
    if (this.props.isOpen) classnames += ' nav-background--open'
    if (this.state.isOpaque) classnames += ' nav-background--opaque'
    return (
      <div className={ classnames }>
        { this.props.children }
      </div>
    )
  }
}

const { node, bool, func } = PropTypes

NavBackground.propTypes = {
  children: node,
  isOpaque: bool,
  isOpen: bool,
  setActiveSection: func.isRequired
}
